// LogSystem.h
#ifndef LOG_SYSTEM_H
#define LOG_SYSTEM_H

#include "llvm/Support/raw_ostream.h"
#include <string>
#include <map>
#include <set>
#include <sstream>

namespace logging {

enum class LogLevel {
    TRACE,
    DEBUG,
    INFO,
    WARNING,
    ERROR,
    NONE
};

class LogSystem {
public:
    static LogSystem& getInstance() {
        static LogSystem instance;
        return instance;
    }

    void setGlobalLevel(LogLevel level) {
        globalLevel = level;
    }

    void setFunctionLevel(const std::string& funcName, LogLevel level) {
        functionLevels[funcName] = level;
    }

    void enableFunction(const std::string& funcName) {
        enabledFunctions.insert(funcName);
    }

    void disableFunction(const std::string& funcName) {
        enabledFunctions.erase(funcName);
    }

    void setContextFunction(const std::string& funcName) {
        currentFunctionName = funcName;
    }

    bool shouldLog(const std::string& funcName, LogLevel level) {
        if (enabledFunctions.find(funcName) == enabledFunctions.end()) {
            return false;
        }

        auto it = functionLevels.find(funcName);
        if (it != functionLevels.end()) {
            return level >= it->second;
        }

        return level >= globalLevel;
    }

    template<typename... Args>
    void log(const std::string& funcName, LogLevel level, const char* format, Args... args) {
        if (!shouldLog(funcName, level)) {
            return;
        }

        std::string levelStr;
        switch (level) {
            case LogLevel::TRACE:   levelStr = "TRACE"; break;
            case LogLevel::DEBUG:   levelStr = "DEBUG"; break;
            case LogLevel::INFO:    levelStr = "INFO"; break;
            case LogLevel::WARNING: levelStr = "WARNING"; break;
            case LogLevel::ERROR:   levelStr = "ERROR"; break;
            default: return;
        }

        std::string msg(format);
        replaceAll(msg, args...);

        llvm::errs() << "[" << levelStr << "]";
        if (!currentFunctionName.empty()) {
            llvm::errs() << "[" << currentFunctionName << "]";
        }
        llvm::errs() << "[" << funcName << "] " << msg << "\n";
    }

private:
    LogSystem() : globalLevel(LogLevel::INFO) {}
    LogLevel globalLevel;
    std::map<std::string, LogLevel> functionLevels;
    std::set<std::string> enabledFunctions;
    std::string currentFunctionName;

    // 递归终止条件
    void replaceAll(std::string&) {}

    // 递归替换函数
    template<typename T, typename... Args>
    void replaceAll(std::string& str, T&& value, Args&&... args) {
        std::stringstream ss;
        ss << std::forward<T>(value);
        size_t pos = str.find("{0}");
        if (pos != std::string::npos) {
            str.replace(pos, 3, ss.str());
        }
        // 移除已处理的参数，继续处理剩余参数
        std::string rest = str;
        size_t start = 0;
        while ((pos = rest.find("{", start)) != std::string::npos) {
            size_t end = rest.find("}", pos);
            if (end != std::string::npos) {
                std::string num = rest.substr(pos + 1, end - pos - 1);
                int n = std::stoi(num);
                if (n > 0) {  // 如果不是 {0}
                    rest.replace(pos, end - pos + 1, "{" + std::to_string(n-1) + "}");
                }
            }
            start = pos + 1;
        }
        str = rest;
        replaceAll(str, std::forward<Args>(args)...);
    }
};

#define LOG_TRACE(func, ...) \
    logging::LogSystem::getInstance().log(func, logging::LogLevel::TRACE, __VA_ARGS__)
#define LOG_DEBUG(func, ...) \
    logging::LogSystem::getInstance().log(func, logging::LogLevel::DEBUG, __VA_ARGS__)
#define LOG_INFO(func, ...) \
    logging::LogSystem::getInstance().log(func, logging::LogLevel::INFO, __VA_ARGS__)
#define LOG_WARNING(func, ...) \
    logging::LogSystem::getInstance().log(func, logging::LogLevel::WARNING, __VA_ARGS__)
#define LOG_ERROR(func, ...) \
    logging::LogSystem::getInstance().log(func, logging::LogLevel::ERROR, __VA_ARGS__)

} // namespace logging

#endif // LOG_SYSTEM_H